	/*****************************************
	 * Szukanie drogi w grafie algorytmem A* *
	 *           Maciej Czech 1996           *
	 *              Prolog 1.0               *
	 *****************************************/


/* Glowny algorytm szukania drogi w grafie:

	1. Utworz liste wezlow otwartych OPEN i wloz do tej listy wezel
	   startowy oznaczony symbolem 's'.

	2. Stworz liste wezlow zamknietych zwana CLOSED, ktora poczatkowo
	   jest pusta.

	3. PETLA: jezeli lista OPEN jest pusta to koniec bez sukcesu.

	4. Wez pierwszy element listy OPEN, usun go z OPEN i umiesc go w
	   liscie CLOSED. Ten element oznaczony bedzie jako 'n'.

	5. Jezeli 'n' jest wezlem docelowym to wyjdz z sukcesem. Rozwiazaniem
	   jest trasa od 's' do 'n'.

	6. Rozwin wezel 'n', generujac zbior 'M' jego nastepcow w grafie.
	   Zapamietaj ich jako nastepcy wezla 'n'.

	7. Ustaw relacje nastepstwa po wezle 'n' dla tych wezlow zbioru 'M',
	   ktore nie byly do tej pory ani w liscie OPEN, ani w liscie CLOSED.
	   Dodaj te wezly do listy OPEN. A dla kazdego elementu zbioru 'M',
	   ktory byl juz w liscie OPEN lub CLOSED zadecyduj, czy nie trzeba
	   zmienic relacji poprzednictwa na wezel 'n'. Dla kazdego elementu
	   ze zbioru 'M', ktory nalezal juz do listy CLOSED zadecyduj, dla
	   kazdego z jego nastepcow, czy nie trzeba zmienic relacji nastepstwa.

	8. Uporzadkuj liste OPEN ze wzgledu na arbitralnie przyjeta funkcje.

	9. Idz do 'PETLA' (pkt. 3).
*/


/* Domeny:
   odleglosc: - 	uzywana jako odleglosc miasta od miasta, jako wartosc
                	funkcji celu, jako dlugosc drogi do celu;
   wspolrzedna_x/y: -	uzyte w predykacie "polozenie" jako wspolrzedne
			graficzne danego miasta na mapie;
   miasto: -	symboloczna nazwa miasta;
   trasa: -	lista miast;
*/
domains
	odleglosc = real
	wspolrzedna_x, wspolrzedna_y = real
	miasto = symbol
	trasa = miasto*
	
/* Zadeklarowany "xnode" predykat sluzacy do zapamietywania w pamieci RAM 
   wezla wraz z jego poprzednikiem (rodzicem w rozwijanym drzewie), oraz 
   aktualnej wartosci funkcji celu. 
*/
database
	xnode(miasto,miasto,odleglosc)
		
predicates
	clear_n
	to_begining(miasto,odleglosc)
	construct(miasto,trasa,trasa,odleglosc,odleglosc)
	open_not_empty(trasa)
	check_closed(miasto,miasto,miasto,trasa)
	check_open(miasto,miasto,miasto,trasa)
	add_open(miasto,miasto,miasto,trasa,trasa,trasa)
	fc(miasto,miasto,miasto,odleglosc)
	change_next(miasto,miasto)
	sprawnosc(trasa,odleglosc)
	node(miasto,miasto,odleglosc)
	length(trasa,odleglosc)
	reverse(trasa,trasa)
	revappend(trasa,trasa,trasa)
	zapisz_liste(miasto,trasa,miasto,trasa,trasa,trasa)
	eq(trasa,trasa)
	droga(miasto,miasto,odleglosc)
	przejazd(miasto,miasto,odleglosc)
	polozenie(miasto,wspolrzedna_x,wspolrzedna_y)
	member(miasto,trasa)
	wstaw(miasto,trasa,trasa)
	lepiej(miasto,miasto)
	uporzadkuj(trasa,trasa)
	expand(miasto,miasto,trasa,trasa,trasa)
	szukaj_trasy(miasto,trasa,trasa)
	writelist(trasa)
	plan(miasto,miasto,trasa,odleglosc)
	okno
	prawda(miasto)
	dial(real)

/* Wewnetrzny cel:
*/
goal
	okno.

/* Czesc definicyjna klauzuli:
*/
clauses

/* Definicja podstawienia:
   "eq(X,X)"
   X - listy miast
*/
eq(X,X).

/* Wypisywanie listy miast. Uzywana do wyswietlenia wyniku poszukiwan w
   grafie:
   "writelist(X)"
   X - lista miast
*/
writelist([]).
writelist([X|XT]):-
	write("      "),
	write(X),
	nl,
	writelist(XT).

/* Definiowanie istnienia drogi przejazdu jako relacji symetrycznej:
   "przejazd(X,Y,D)"
   X - miasto pierwsze
   Y - miasto drugie
   D - dlugosc drogi pomiedzy X i Y
*/
przejazd(X,Y,D) if droga(X,Y,D).
przejazd(X,Y,D) if droga(Y,X,D).


/* Definicja przynaleznosci do listy:
   "member(X,Y)"
   X - miasto nalezace do listy miast Y
*/
member(X,[X|_]).
member(X,[_|T]):-
	member(X,T).

/* Wstawianie miasta do listy miast z zachowaniem kolejnosci - miasto
   najlepsze w sensie nizej zdefiniowanej funkcji celu jest pierwsze na
   liscie, najgorsze - ostatnie. Uzywane do porzadkowania listy OPEN -
   listy wezlow otwartych:
   "wstaw(X,Y,Z)"
   X - miasto do wstawienia do listy Y
   Z - wynik operacji (lista Y ze wstawionym miastem X)
*/
wstaw(X,[],[X]).
wstaw(X,[Y|YT],[Y|ZT]):-
	lepiej(Y,X),
	!,
	wstaw(X,YT,ZT).
wstaw(X,[Y|YT],[X,Y|YT]):-
	lepiej(X,Y).

/* Definicja relacji "lepiej" pomiedzy miastami "X" i "Y" w sensie wartosci
   funkcji celu nizej zadeklarowanej (suma dlugosci juz przebytej drogi do
   danego wezla i odleglosci od celu w sensie geometrycznym):
   "lepiej(X,Y)"
   X - miasto lepsze od miasta Y
*/
lepiej(X,Y):-
	node(_,X,FX),
	node(_,Y,FY),
	FX<=FY.

/* Porzadkowanie listy miast-kandydatow do rozwijania wzgledem wartosci funkcji
   celu. Pierwszy na liscie jest najlepszy (najmniejsza wartosc funkcji celu):
   "uporzadkuj(X,Y)"
   X - lista do uporzadkowania
   Y - lista uporzadkowana
*/
uporzadkuj([X],[X]).
uporzadkuj([X|XT],LU):-
	uporzadkuj(XT,U),
	wstaw(X,U,LU).

/* Wyszukiwanie sasiadow, miasta "Miasto" zapisanie w pamieci relacji 
   poprzedzania wygenerowanych miast i okreslanie kolejnosci miast do dalszego
   badania. Uzyty predykat niedeterministyczny "findall":
   "expand(A,B,C,D,E)"
   A - miasto, ktore bedzie rozwijane (ktore jest poprzednikiem dla miast
       bedacych rozwinieciem),
   B - miasto celu poszukiwan,
   C - aktualna (przed rozwinieciem) lista wezlow otwartych - czyli tych, ktore
       nie sa dla nikogo poprzednikami w grafie,
   D - zwracana uaktualniona lista wezlow otwartych,
   E - aktualna lista wezlow zamknietych - czyli tych, ktore sa juz dla kogos
       poprzednikami. Ta lista nie jest tu zmieniana.
*/
expand(Miasto,Cel,Open,NewOpen,Closed):-
	findall(Xkandydat,przejazd(Miasto,Xkandydat,_),ListaKandydatow),
	zapisz_liste(Miasto,ListaKandydatow,Cel,Open,NewOpen,Closed).

/* Predykat zapisujacy wygenerowane w "expand" kolejne elementy listy
   nastepnikow jakiegos miasta do pamieci RAM wraz z ich wartosciami funkcji
   celu, jezeli kazdy z nich przeszedl pomyslnie testy przynaleznosci do
   listy zamknietej i listy otwartej. W wykorzystanych predykatach:
   "check_closed", "check_open" jest mozliwa modyfikacja aktualnej bazy wiedzy
   w pamieci RAM w zaleznosci od wyniku porownania wartosci funkcji celu
   obliczonej z wykorzystaniem aktualnego poprzednika z wartoscia funkcji celu
   dla danego wezla zapamietana w pamieci. Predykat "add_open" dodaje do listy
   wezlow otwartych kolejny element wygenerowanych nastepnikow, jesli jeszcze
   go w tej liscie nie ma. Przy okazji dodawane sa te miasta do bazy wiedzy w
   pamieci:
   "zapisz_liste(A,B,C,D,E,F)"
   A - miasto, ktore bylo rozwijane (poprzednik dla wszystkich elementow listy
       B),
   B - lista wygenerowanych nastepnikow
   C - miasto celu
   D - lista wezlow otwartych przed wywolaniem
   E - zwracana, odpowiednio zmodyfikowana lista wezlow otwartych
   F - lista wezlow zamknietych
*/
zapisz_liste(_,[],_,Open,Open,_):-!.
zapisz_liste(M,[H|T],Cel,Open,NOpen,Closed):-
	check_closed(M,H,Cel,Closed),
	check_open(M,H,Cel,Open),
	add_open(M,H,Cel,Open,Closed,NewOpen),
	zapisz_liste(M,T,Cel,NewOpen,NOpen,Closed).

/* Predykat dodajacy do listy wezlow otwartych element wygenerowany w "expand"
   jesli jeszcze on nie istnieje ani w liscie wezlow otwartych, ani w liscie
   wezlow zamknietych. Obliczana jest tez dla niego wartosc funkcji celu i
   zapamietywana w pamieci:
   "add_open(A,B,C,D,E,F)"
   A - miasto bedace poprzednikiem dla miasta B (miasto rozwijane),
   B - miasto, ktore ma byc dodawane do listy wezlow otwartych,
   C - miasto docelowe,
   D - lista wezlow otwartych przed ewentualna modyfikacja,
   E - lista wezlow zamknietych,
   F - zwracana lista zmodyfikowana
*/
add_open(M,H,Cel,Open,Closed,[H|Open]):-
	not(member(H,Open)),
	not(member(H,Closed)),
	fc(H,M,Cel,F),
	assertz(xnode(M,H,F)),
	!.
add_open(_,H,_,Open,_,Open):-
	member(H,Open),
	!.
add_open(_,H,_,Open,Closed,Open):-
	member(H,Closed),
	!.
	
/* Predykat sprawdzajacy, czy miasto "Node" wygenerowane w rozwijaniu wezla
   "Parent" nalezy do listy wezlow zamknietych. Jesli tak, to obliczana jest
   wartosc funkcji celu i porownywana jest z wartoscia zapamietana w pamieci.
   Jesli obliczona wartosc jest lepsza (mniejsza), to dokonuje sie modyfikacji
   bazy wiedzy w ten spoob, ze dla wezla "Node", ktorego wartosc funkcji celu
   byla obliczana, zmieniany jest poprzednik (rodzic) i wartosc funkcji celu
   na ta obliczona. Nastepnie predykat "change_next" modyfikuje wartosci
   funkcji celu we wszystkich wezlach "dzieciach" - bedacych kolejnymi
   nastepnikami wezla "Node":
   "check_closed(A,B,C,D)"
   A - miasto poprzedzajace miasto B,
   B - miasto nalezace do listy wezlow zamknietych,
   C - miasto docelowe,
   D - lista wezlow zamknietych
*/
check_closed(Parent,Node,Cel,Closed):-
	member(Node,Closed),
	fc(Node,Parent,Cel,F),
	node(XParent,Node,Fm),
	F<Fm,
	retract(xnode(XParent,Node,Fm)),
	assertz(xnode(Parent,Node,F)),
	change_next(Node,Cel).	
check_closed(Parent,Node,Cel,Closed):-
	member(Node,Closed),
	fc(Node,Parent,Cel,F),
	node(_,Node,Fm),
	F>=Fm.
check_closed(Parent,Node,_,Closed):-
	member(Node,Closed),
	node(Node,Parent,_).
check_closed(_,Node,_,Closed):-
	member(Node,Closed),
	not(node(_,Node,_)).
check_closed(_,Node,_,Closed):-
	not(member(Node,Closed)),
	!.
/* Predykat sprawdzjacy czy miasto "Node" nalezy do listy wezlow otwartych.
   Jesli tak, to obliczana jest wartosc funkcji celu i porownywana jest z
   wartoscia zapamietana w pamieci. Jesli obliczona wartosc jest lepsza
   (mniejsza), to dokonuje sie modyfikacji bazy wiedzy w ten spoob, ze dla
   wezla "Node", ktorego wartosc funkcji celu byla obliczana, zmieniany jest
   poprzednik (rodzic) i wartosc funkcji celu na aktualnie obliczona:
   "check_open(A,B,C,D)"
   A - miasto poprzedzajace miasto B,
   B - miasto nalezace do listy wezlow otwartych,
   C - miasto docelowe,
   D - lista wezlow otwartych
*/
check_open(Parent,Node,Cel,Open):-
	member(Node,Open),
	fc(Node,Parent,Cel,F),
	node(XParent,Node,Fm),
	F<Fm,
	retract(xnode(XParent,Node,Fm)),
	assertz(xnode(Parent,Node,F)).
check_open(Parent,Node,Cel,Open):-
	member(Node,Open),
	fc(Node,Parent,Cel,F),
	node(_,Node,Fm),
	F>=Fm.	
check_open(_,Node,_,Open):-
	not(member(Node,Open)),
	!.

/* Predykat obliczajacy wartosc funkcji celu dla zadanego wezla "Node", z
   rodzicem (poprzednikiem) "Parent". Funkcja celu jest suma dlugosci drogi
   do tej pory wygenerowanej i funkcji heurystycznej, ktora jest geometryczna
   odlegloscia tego miasta "Node" do miasta docelowego. Poniewaz wspolrzedne
   geometryczne sa podane w skali "mikro" to konieczne jest zastosowanie
   wspolczynnika skalujacego o wartosci 53.00:
   "fc(A,B,C,D)"
   A - wezel, dla ktorego obliczana jest wartosc funkcji celu,
   B - wezel poprzedni dla miasta A,
   C - miasto docelowe,
   D - zwracana wartosc funkcji celu
*/
fc(Node,Parent,Cel,F):-
	not(node(_,Parent,_)),
	przejazd(Node,Parent,D),
	polozenie(Node,Xn,Yn),
	polozenie(Cel,Xc,Yc),
	F=D+(sqrt((Xn-Xc)*(Xn-Xc)+(Yn-Yc)*(Yn-Yc)))*40,
	!.
fc(Node,Parent,Cel,F):-
	node(_,Parent,_),
	przejazd(Node,Parent,D),
	polozenie(Node,Xn,Yn),
	polozenie(Cel,Xc,Yc),
	Fo=D+(sqrt((Xn-Xc)*(Xn-Xc)+(Yn-Yc)*(Yn-Yc)))*53,
	to_begining(Parent,ND),
	F=Fo+ND,
	!.

/* Predykat obliczajacy dlugosc drogi do wezla startowego z zadanego wezla:
   "to_begining(A,B)"
   A - miasto, dla ktorego obliczamy droge do poczatku
   B - zwracana wartosc drogi
*/ 
to_begining(Node,0):-
	not(node(_,Node,_)).
to_begining(Node,Dlugosc):-
	node(Parent,Node,_),
	przejazd(Parent,Node,D),
	!,
	to_begining(Parent,Dl),
	Dlugosc=Dl+D.

/* Predykat zmieniajacy w bazie wiedzy wartosci funkcji celu nastepnikow wezla
   "Node" na aktualne. Predykat wywolywany, gdy zmienila sie kolejnosc
   nastepstw wezlow bedacych w liscie wezlow zamknietych:
   "change_next(A,B)"
   A - miasto-wezel, od ktorego maja byc dokonywane zmiany,
   B - miasto docelowe
*/
change_next(Node,Cel):-
	node(Node,NextNode,Fm),
	fc(NextNode,Node,Cel,F),
	retract(xnode(Node,NextNode,Fm)),
	assertz(xnode(Node,NextNode,F)),
	change_next(NextNode,Cel).
change_next(Node,_):-
	not(node(Node,_,_)),
	!.

/* Predykat odwolujacy sie do bazy wiedzy:
   "node(A,B,C)"
   A - rodzic miasta B,
   B - miasto, dla ktorego jest obliczona funkcja celu C,
   C - wartosc funkcji celu dla miasta B
*/
node(Parent,Node,Function):-
	xnode(Parent,Node,Function).


/* Predykat generujacy trase z aktualnej bazy wiedzy z miasta koncowego do
   poczatkowego. Predykat korzysta z informacji o poprzednikach. Generujac
   oblicza rowniez dlugosc generowanej trasy:
   "construct(A,B,C,D,E)"
   A - miasto docelowe (startowe dla rekonstrukcji trasy),
   B - posrednia (poczatkowa) lista miast - przy wywolaniu predykatu lista
       ta musi zawierac miasto A (B=[A]),
   C - zwracany wynik rekonstrukcji trasy - lista miast,
   D - posrednia (poczatkowa) wartosc dlugosci drogi - przy wywolaniu D=0,
   E - zwracana dlugosc trasy
*/
construct(Cel,Trasa,NowaTrasa,Dlugosc,NowaDlugosc):-
	node(Parent,Cel,_),
	przejazd(Parent,Cel,D),
	ND=Dlugosc+D,
	construct(Parent,[Parent|Trasa],NowaTrasa,ND,NowaDlugosc).
construct(Cel,Trasa,NowaTrasa,Dlugosc,NowaDlugosc):-
	not(node(_,Cel,_)),
	eq(NowaTrasa,Trasa),
	NowaDlugosc=Dlugosc,
	!.

/* Obliczanie ilosci elementow w liscie:
   "length(A,B)"
   A - badana lista,
   B - wynik obliczen
*/
length([],0).
length([_|List],N):-
	length(List,N1),
	N=N1+1.

	
/* Odwracanie listy:
   "reverse(A,B)"
   A - lista do odwrocenia,
   B - lista odwrocona
*/
reverse(L1,L2):-
	revappend(L1,[],L2).
	
/* Doklejanie do listy w odwrotnej kolejnosci. */
revappend([],L,L).
revappend([X|L1],L2,L3):-
	revappend(L1,[X|L2],L3).

/* Czyszczenie pamieci z zapamietanej bazy wiedzy "xnode".
*/
clear_n:-
	retract(xnode(_,_,_)),
	fail.
clear_n.

/* Sprawdzanie, czy lista wezlow otwartych nie jest pusta:
   "open_not_empty(A)"
   A - sprawdzana lista weslow otwartych
*/
open_not_empty(Open):-
	length(Open,L),
	L>0.

/* Predykat obliczajacy stosunek ilosci wezlow w otrzymanej trasie do ilosci
   wszystkich wezlow:
   "sprawnosc(A,B)"
   A - lista miast reprezentujaca trase,
   B - zwracany wynik wspolczynnika
*/
sprawnosc(Trasa,W):-
	length(Trasa,T),
	findall(X,node(_,X,_),Y),
	length(Y,P),
	W=T/P.

/* Glowna petla porgramowa: wygenerowanie w pamieci RAM spojnej bazy wiedzy
   o wygenerowanych wezlach, i ich poprzednikach w rozwinietym drzewie. W bazie
   tej bedzie istniec tylko jedna sciezka krytyczna od miasta poczatkowego do
   miasta koncowego:
   "szukaj_trasy(A,B,C)"
   A - miasto docelowe,
   B - lista wezlow otwartych (w pierwszym wywolaniu zawiera tylko miasto
       startowe poszukiwan),
   C - lista wezlow zamknietych (w pierwszym wywolaniu lista jest pusta)
*/
szukaj_trasy(Cel,[HO|_],_):-
	HO=Cel,
	!.
szukaj_trasy(Cel,[HO|TO],Closed):-
	open_not_empty([HO|TO]),
	expand(HO,Cel,TO,NewOpen,[HO|Closed]),
	uporzadkuj(NewOpen,ReorderdOpen),
	szukaj_trasy(Cel,ReorderdOpen,[HO|Closed]).

/* Wypisywanie listy miast */
okno:-
makewindow(1,1,2," Program poszukiwania heurystycznego ",0,0,25,80),
cursor(8,11),
write("Program sluzy do poszukiwania polaczenia kolejowego "),
cursor(9,14),
write("pomiedzy dwoma dowolnymi miastami wojewodzkimi "),
cursor(11,15),
write(" Szukana trasa znajdywana jest algorytmem 'A*' "),
cursor(15,30),
write("PRESS ANY KEY"),
readchar(_),
removewindow,
makewindow(3,32,88," Lista miast ",0,0,20,80),
write("     "),nl,
write("  1.biala_podlaska           17.krakow                   33.radom    "),nl,
write("  2.bialystok                18.krosno                   34.rzeszow  "),nl,
write("  3.bielsko_biala            19.legnica                  35.siedlce "),nl,
write("  4.bydgoszcz                20.leszno                   36.sieradz "),nl,
write("  5.chelm                    21.lublin                   37.skierniewice"),nl,
write("  6.ciechanow                22.lomza                    38.slupsk  "),nl,
write("  7.czestochowa              23.lodz                     39.suwalki "),nl,
write("  8.elblag                   24.nowy_sacz                40.szczecin  "),nl,
write("  9.gdansk                   25.olsztyn                  41.tarnobrzeg "),nl,
write(" 10.gorzow_wielkopolski      26.opole                    42.tarnow   "),nl,
write(" 11.jelenia_gora             27.ostroleka                43.torun "),nl,
write(" 12.kalisz                   28.pila                     44.walbrzych "),nl,
write(" 13.katowice                 29.piotrkow_trybunalski     45.warszawa  "),nl,
write(" 14.kielce                   30.plock                    46.wloclawek "),nl,
write(" 15.konin                    31.poznan                   47.wroclaw "),nl,
write(" 16.koszalin                 32.przemysl                 48.zamosc "),nl,
write(" -----------                 -------------               49.zielona_gora"),

/* Wczytywanie danych wejsciowych */
makewindow(2,15,7," DANE WEJSCIOWE ",20,0,5,80),
write("Miasto poczatkowe :"),
field_attr(0,0,18,112),
readln(Miasto),
prawda(Miasto),
write("Miasto koncowe :"),
field_attr(1,0,15,112),
readln(Cel),
prawda(Cel),
shiftwindow(2),
removewindow,
shiftwindow(3),
removewindow,
plan(Miasto,Cel,_,_).


/* Inicjacja poszukian */

plan(Miasto,Cel,Trasa,Dystans):-
	szukaj_trasy(Cel,[Miasto],[]),
	construct(Cel,[Cel],Trasa,0,Dystans),
	sprawnosc(Trasa,W),
	clear_n,
	makewindow(4,48,56," Wynik poszukiwan ",0,0,15,80),
	write(" Znaleziona trasa:"),nl,
	writelist(Trasa),
	write(" Dlugosc trasy: "),
	write(Dystans),
	write(" km"),
	cursor(2,40),
	write("Sprawnosc algorytmu: "),
	write(W),
		
/* Dialog z uzytkownikiem */
	makewindow(5,7,7," DIALOG ",15,0,10,80),
     	write(" CZY PRACUJEMY DALEJ?  tak PRESS 1"),nl,
     	write("                       nie PRESS 0"),nl,
     	field_attr(0,0,20,112),
     	field_attr(0,21,15,15),
     	field_attr(1,21,15,15),
     	readreal(Z),
	shiftwindow(4),
	removewindow,
	shiftwindow(5),
	removewindow,
     	!,
     	dial(Z),
	!.

dial(P):-NIE=0,P>NIE,okno,!.
dial(_).

/* Sprawdzanie danych wejsciowych z baza danych */
prawda(T):-polozenie(T,_,_),!.
prawda(_):-makewindow(7,7,240," KOMUNIKAT ",15,51,10,29),
	   write(" "),nl,
	   write("   NIE MA TAKIEGO MIASTA"),nl,
	   write("       PRESS ANY KEY"),
	   sound(20,392),
	   sound(20,349),
	   sound(20,466),
	   sound(10,523),
	   sound(10,698),
	   sound(20,587),
	   sound(10,523),
	   sound(10,698),
           sound(20,587),
           sound(20,466),
           sound(20,523),
           sound(20,392),
           sound(20,349),
           readchar(_),
	   removewindow,
           okno.


/* Baza danych programu */
     droga(biala_podlaska,siedlce,83).
     droga(biala_podlaska,lublin,166).
     droga(bialystok,suwalki,140).
     droga(bialystok,lomza,102).
     droga(bialystok,siedlce,160).
     droga(bielsko_biala,krakow,100).
     droga(bielsko_biala,katowice,54).
     droga(bielsko_biala,nowy_sacz,169).
     droga(bydgoszcz,gdansk,160).
     droga(bydgoszcz,elblag,175).
     droga(bydgoszcz,torun,50).
     droga(bydgoszcz,konin,177).
     droga(bydgoszcz,poznan,134).
     droga(bydgoszcz,pila,87).
     droga(chelm,lublin,72).
     droga(chelm,zamosc,83).
     droga(ciechanow,olsztyn,133).
     droga(ciechanow,warszawa,93).
     droga(ciechanow,plock,160).
     droga(ciechanow,torun,187).
     droga(czestochowa,piotrkow_trybunalski,86).
     droga(czestochowa,kielce,122).
     droga(czestochowa,katowice,88).
     droga(czestochowa,opole,96).
     droga(czestochowa,sieradz,120).
     droga(elblag,olsztyn,143).
     droga(elblag,torun,167).
     droga(elblag,gdansk,79).
     droga(gdansk,slupsk,131).
     droga(gorzow_wielkopolski,szczecin,144).
     droga(gorzow_wielkopolski,pila,117).
     droga(gorzow_wielkopolski,poznan,143).
     droga(gorzow_wielkopolski,zielona_gora,118).
     droga(jelenia_gora,zielona_gora,221).
     droga(jelenia_gora,legnica,93).
     droga(jelenia_gora,walbrzych,47).
     droga(kalisz,konin,163).
     droga(kalisz,sieradz,50).
     droga(kalisz,opole,155).
     droga(kalisz,wroclaw,129).
     droga(kalisz,leszno,122).
     droga(kalisz,poznan,137).
     droga(katowice,krakow,78).
     droga(katowice,kielce,170).
     droga(katowice,opole,96).
     droga(kielce,radom,85).
     droga(kielce,tarnobrzeg,164).
     droga(kielce,krakow,132).
     droga(kielce,piotrkow_trybunalski,197).
     droga(konin,wloclawek,134).
     droga(konin,plock,132).
     droga(konin,sieradz,140).
     droga(konin,poznan,100).
     droga(koszalin,slupsk,67).
     droga(koszalin,pila,158).
     droga(koszalin,szczecin,160).
     droga(krakow,tarnow,78).
     droga(krosno,nowy_sacz,100).
     droga(krosno,tarnow,126).
     droga(krosno,rzeszow,93).
     droga(krosno,przemysl,145).
     droga(legnica,zielona_gora,137).
     droga(legnica,wroclaw,65).
     droga(legnica,walbrzych,78).
     droga(leszno,wroclaw,96).
     droga(leszno,poznan,69).
     droga(leszno,zielona_gora,100).
     droga(lublin,zamosc,117).
     droga(lublin,tarnobrzeg,131).
     droga(lublin,radom,129).
     droga(lublin,siedlce,139).
     droga(lomza,ostroleka,42).
     droga(lodz,skierniewice,66).
     droga(lodz,piotrkow_trybunalski,65).
     droga(lodz,sieradz,62).
     droga(lodz,plock,121).
     droga(nowy_sacz,tarnow,88).
     droga(olsztyn,suwalki,225).
     droga(olsztyn,ostroleka,138).
     droga(olsztyn,torun,162).
     droga(opole,wroclaw,82).
     droga(ostroleka,warszawa,115).
     droga(ostroleka,siedlce,125).
     droga(pila,poznan,96).
     droga(pila,slupsk,176).
     droga(piotrkow_trybunalski,skierniewice,77).
     droga(piotrkow_trybunalski,radom,154).
     droga(plock,warszawa,177).
     droga(plock,skierniewice,120).
     droga(plock,wloclawek,108).
     droga(poznan,zielona_gora,137).
     droga(przemysl,rzeszow,87).
     droga(przemysl,zamosc,207).
     droga(radom,warszawa,105).
     droga(radom,siedlce,167).
     droga(radom,tarnobrzeg,161).
     droga(radom,skierniewice,155).
     droga(rzeszow,tarnow,80).
     droga(rzeszow,tarnobrzeg,70).
     droga(rzeszow,zamosc,208).
     droga(siedlce,warszawa,96).
     droga(skierniewice,warszawa,66).
     droga(tarnobrzeg,tarnow,104).
     droga(torun,wloclawek,54).
     droga(walbrzych,wroclaw,80).
     
     polozenie(biala_podlaska,14.3,6.9).
     polozenie(bialystok,14.3,9.8).
     polozenie(bielsko_biala,7.4,0.4).
     polozenie(bydgoszcz,5.7,9.6).
     polozenie(chelm,15.2,4.4).
     polozenie(ciechanow,10.1,8.9).
     polozenie(czestochowa,7.5,3.3).
     polozenie(elblag,8.0,12.6).
     polozenie(gdansk,6.8,13.1).
     polozenie(gorzow_wielkopolski,1.0,8.8).
     polozenie(jelenia_gora,1.6,3.6).
     polozenie(kalisz,5.7,6.0).
     polozenie(katowice,7.3,1.7).
     polozenie(kielce,10.2,3.4).
     polozenie(konin,6.0,7.1).
     polozenie(koszalin,2.7,12.7).
     polozenie(krakow,9.0,1.2).
     polozenie(krosno,12.2,0.1).
     polozenie(legnica,2.3,4.5).
     polozenie(leszno,3.1,6.2).
     polozenie(lublin,13.5,4.6).
     polozenie(lomza,12.4,9.9).
     polozenie(lodz,8.1,5.9).
     polozenie(nowy_sacz,10.4,0).
     polozenie(olsztyn,9.7,11.5).
     polozenie(opole,5.4,3.0).
     polozenie(ostroleka,11.5,9.5).
     polozenie(pila,3.5,9.8).
     polozenie(piotrkow_trybunalski,8.5,5.0).
     polozenie(plock,8.5,8.1).
     polozenie(poznan,3.8,7.7).
     polozenie(przemysl,14.0,0.6).
     polozenie(radom,11.0,4.9).
     polozenie(rzeszow,12.6,1.2).
     polozenie(siedlce,12.9,7.2).
     polozenie(sieradz,6.8,5.4).
     polozenie(skierniewice,9.3,6.5).
     polozenie(slupsk,4.2,13.4).
     polozenie(suwalki,13.8,12.6).
     polozenie(szczecin,0,10.7).
     polozenie(tarnobrzeg,12.0,2.7).
     polozenie(tarnow,10.8,1.1).
     polozenie(torun,6.6,9.4).
     polozenie(walbrzych,2.6,3.2).
     polozenie(warszawa,10.7,7.2).
     polozenie(wloclawek,7.3,8.3).
     polozenie(wroclaw,3.8,4.1).
     polozenie(zamosc,14.7,3.2).
     polozenie(zielona_gora,1.4,6.5).
