/* PLAN_F optymalizacja trasy i zliczanie dlugosci  */
/* w petli typu "nawrot - fail"                     */

DOMAINS
     odleglosc = real
     wspolrzedna_x, wspolrzedna_y = real
     miasto = symbol
     trasa  = miasto*
     
DATABASE
     limit(odleglosc)
     trasa(trasa)     

PREDICATES
              eq(trasa,trasa)
nondeterm     droga(miasto,miasto,odleglosc)
nondeterm     przejazd(miasto,miasto,odleglosc)
nondeterm     wliscie(miasto,trasa)
nondeterm     szukaj_trasy(miasto,miasto,trasa,trasa,
                  odleglosc,odleglosc,odleglosc)
              writelist(trasa)
nondeterm     plan(miasto,miasto,trasa,odleglosc,odleglosc)
nondeterm     plan_f(miasto,miasto,trasa,odleglosc,odleglosc)
              zamien(trasa,trasa)


CLAUSES

/* Definicja podstawiania */
eq(X,X).


/* Wypisywanie listy miast */
writelist([]).
writelist([X|XT]):- write(X),nl,writelist(XT).


/* Definiowanie istnienia drogi przjazdu
        jako relacji symetrycznej        */
przejazd(X,Y,D) if droga(X,Y,D).
przejazd(X,Y,D) if droga(Y,X,D).

/* Definicja przynaleznosci do listy */
wliscie(X,[X|_]).
wliscie(X,[_|T]) if wliscie(X,T).




/***     Glowna petla programowa     ***
 * wyszukiwanie trasy i zliczanie odleglosci */

szukaj_trasy(Cel,Cel,Trasa,Wynik,Dlugosc,Dystans,_):-
     eq(Trasa,Wynik),
     Dystans=Dlugosc.
szukaj_trasy(Miasto,Cel,Trasa,Wynik,Dlugosc,Dystans,Ograniczenie):-
     przejazd(Miasto,Kandydat,D),
     not(wliscie(Kandydat,Trasa)),
     NowaDlugosc=Dlugosc+D,
/* */     limit(G),
/* */     NowaDlugosc < G,
     NowaDlugosc < Ograniczenie,
     szukaj_trasy(Kandydat,Cel,[Kandydat|Trasa],Wynik,
                  NowaDlugosc,Dystans,Ograniczenie).


/*  Inicjacja poszukiwan  */
plan(Miasto,Cel,Wynik,Dystans,Ograniczenie):-
     eq(Trasa,[Cel]),
     szukaj_trasy(Cel,Miasto,Trasa,Wynik,0,Dystans,Ograniczenie).


/* poszukiwanie rozwiazania optymalnego metoda ograniczania (B&B) */
/* technika petli z fail */

plan_f(Miasto,Cel,_,_,Ograniczenie):-
     asserta(limit(Ograniczenie)),
     asserta(trasa([])),
     plan(Miasto,Cel,Wynik1,Dystans1,Ograniczenie),
 /*    write(Wynik1,Dystans1),nl,readchar(_),   */
     limit(G),
     Dystans1 < G,
     retract(limit(G)),
     asserta(limit(Dystans1)),
     zamien(_,Wynik1),
     fail.
plan_f(_,_,Wynik,Dystans,_):-
     retract(limit(Dystans)),
     retract(trasa(Wynik)).     

zamien(X,Y):- retract(trasa(X)),asserta(trasa(Y)),!.


/* Baza danych programu */
     droga(krakow,rzeszow,162).
     droga(krakow,lublin,273).
     droga(krakow,kielce,117).
     droga(krakow,lodz,241).
     droga(krakow,katowice,81).
     droga(rzeszow,lublin,168).
     droga(rzeszow,kielce,168).
     droga(lublin,kielce,179).
     droga(lublin,lodz,252).
     droga(lublin,warszawa,161).
     droga(lublin,bialystok,254).
     droga(kielce,lodz,147).
     droga(kielce,warszawa,177).
     droga(kielce,katowice,161).
     droga(kielce,opole,223).
     droga(katowice,opole,100).
     droga(katowice,lodz,193).
     droga(opole,wroclaw,84).
     droga(opole,lodz,184).
     droga(opole,poznan,255).
     droga(wroclaw,lodz,215).
     droga(wroclaw,poznan,171).
     droga(wroclaw,zielona_gora,153).
     droga(zielona_gora,poznan,132).
     droga(zielona_gora,szczecin,217).
     droga(szczecin,poznan,238).
     droga(szczecin,bydgoszcz,258).
     droga(szczecin,koszalin,156).
     droga(koszalin,poznan,242).
     droga(koszalin,bydgoszcz,195).
     droga(koszalin,gdansk,184).
     droga(gdansk,bydgoszcz,174).
     droga(gdansk,warszawa,343).
     droga(gdansk,olsztyn,157).
     droga(olsztyn,bydgoszcz,206).
     droga(olsztyn,lodz,284).
     droga(olsztyn,warszawa,214).
     droga(olsztyn,bialystok,224).
     droga(bialystok,warszawa,194).
     droga(bydgoszcz,poznan,129).
     droga(bydgoszcz,lodz,202).
     droga(bydgoszcz,warszawa,258).
     droga(bydgoszcz,opole,325).
     droga(poznan,lodz,201).
     droga(poznan,warszawa,303).
     droga(lodz,warszawa,133).

