/*Deklaracja dziedzin*/
domains
     odleglosc = real
     wspolrzedna_x, wspolrzedna_y = real
     miasto = symbol
     trasa  = miasto*
     
/*Deklaracja predykatow*/     
predicates
     prawda(miasto)
     okno
     dial(real)
     eq(trasa,trasa)
     droga(miasto,miasto,odleglosc)
     przejazd(miasto,miasto,odleglosc)
     polozenie(miasto,wspolrzedna_x,wspolrzedna_y)
     wliscie(miasto,trasa)
     wstaw(miasto,trasa,trasa,miasto)
     blizej(miasto,miasto,miasto)     
     uporzadkuj(trasa,trasa,miasto)
     ruch(miasto,miasto,miasto)
     szukaj_trasy(miasto,miasto,trasa,trasa,
                  odleglosc,odleglosc)
     writelist(trasa)
     plan(miasto,miasto,trasa,odleglosc)
     
/*Definicja celu*/      
goal
     okno.
     

/*Definicja predykatow*/
clauses

/* Definicja podstawiania */
eq(X,X).     


/* Wypisywanie listy miast */
writelist([]).
writelist([X|XT]):-write("      "),write(X),nl,writelist(XT).


/* Definiowanie istnienia drogi przjazdu
        jako relacji symetrycznej        */
przejazd(X,Y,D) if droga(X,Y,D).
przejazd(X,Y,D) if droga(Y,X,D).

/* Definicja przynaleznosci do listy */     
wliscie(X,[X|_]).
wliscie(X,[_|T]) if wliscie(X,T).
     

     
/* Wstawianie miasta do listy miast
   z zachowaniem kolejnosci - miasto
   najblizsze celu jest pierwsze na 
   liscie, najdalsze - ostatnie     */
wstaw(X,[],[X],_).
wstaw(X,[Y|YT],[Y|ZT],Cel):- blizej(Y,X,Cel),!, wstaw(X,YT,ZT,Cel).
wstaw(X,[Y|YT],[X,Y|YT],Cel):- blizej(X,Y,Cel).
     
/* Definicja odleglosci w sensie geometrycznym 
   i porownywanie odleglosci X i Y od Celu    */
blizej(X,Y,Cel):- polozenie(X,Xx,Yx), polozenie(Y,Xy,Yy), 
                  polozenie(Cel,Xc,Yc), 
                  DX=sqrt((Xc-Xx)*(Xc-Xx)+(Yc-Yx)*(Yc-Yx)),
                  DY=sqrt((Xc-Xy)*(Xc-Xy)+(Yc-Yy)*(Yc-Yy)),
                  DX<=DY.    

/* Porzadkowanie listy miast-kandydatow do 
   rozwijania wzgledem odleglosci od Celu  */      
uporzadkuj([X],[X],_).
uporzadkuj([X|XT],LU,Cel):- uporzadkuj(XT,U,Cel), wstaw(X,U,LU,Cel).      
     
     
/* Wyszukiwanie i okreslanie kolejnosci miast
   do dalszego badania; predykat niedeterminist. */
ruch(Miasto,Cel,Kandydat):-
     findall(Xkandydat,przejazd(Miasto,Xkandydat,_),ListaKandydatow),
     uporzadkuj(ListaKandydatow,ListaUporzadkowana,Cel),
     wliscie(Kandydat,ListaUporzadkowana).
     
/***     Glowna petla programowa     ***
 * wyszukiwanie trasy i zliczanie odleglosci */     
     
szukaj_trasy(Cel,Cel,Trasa,Wynik,Dlugosc,Dystans):- 
     eq(Trasa,Wynik),
     Dystans=Dlugosc.
szukaj_trasy(Miasto,Cel,Trasa,Wynik,Dlugosc,Dystans):-
     ruch(Miasto,Cel,Kandydat),
     not(wliscie(Kandydat,Trasa)),
     przejazd(Miasto,Kandydat,D),
     NowaDlugosc=Dlugosc+D,         
     szukaj_trasy(Kandydat,Cel,[Kandydat|Trasa],Wynik,NowaDlugosc,Dystans).


/* Wypisywanie listy miast */
okno:-
makewindow(1,15,112,"PROGRAM POSZUKIWANIA HEURYSTYCZNEGO",0,0,25,80),
cursor(8,10),
write(" PROGRAM SLUZY DO POSZUKIWANIA POLACZENIA KOLEJOWEGO "),
cursor(9,10),
write(" MIEDZY DWOMA DOWOLNYMI MIASTAMI WOJEWODZKIMI "),nl,
cursor(11,10),
write(" SZUKANA TRASA ZNAJDYWANA JEST METODA PRZESZUKIWANIA WGLAB "),nl,
cursor(15,30),
write("PRESS ANY KEY"),
readchar(_),
makewindow(3,112,7,"LISTA MIAST",0,0,20,80),
write("     "),nl,
write("  1.biala_podlaska           17.krakow                   33.radom    "),nl,
write("  2.bialystok                18.krosno                   34.rzeszow  "),nl,
write("  3.bielsko_biala            19.legnica                  35.siedlce "),nl,
write("  4.bydgoszcz                20.leszno                   36.sieradz "),nl,
write("  5.chelm                    21.lublin                   37.skierniewice"),nl,
write("  6.ciechanow                22.lomza                    38.slupsk  "),nl,
write("  7.czestochowa              23.lodz                     39.suwalki "),nl,
write("  8.elblag                   24.nowy_sacz                40.szczecin  "),nl,
write("  9.gdansk                   25.olsztyn                  41.tarnobrzeg "),nl,
write(" 10.gorzow_wielkopolski      26.opole                    42.tarnow   "),nl,
write(" 11.jelenia_gora             27.ostroleka                43.torun "),nl,
write(" 12.kalisz                   28.pila                     44.walbrzych "),nl,
write(" 13.katowice                 29.piotrkow_trybunalski     45.warszawa  "),nl,
write(" 14.kielce                   30.plock                    46.wloclawek "),nl,
write(" 15.konin                    31.poznan                   47.wroclaw "),nl,
write(" 16.koszalin                 32.przemysl                 48.zamosc "),nl,
write(" -----------                 -------------               49.zielona_gora"),

/* Wczytywanie danych wejsciowych */
makewindow(2,15,7,"DANE WEJSCIOWE",20,0,5,80),
write("miasto poczatkowe :"),
field_attr(0,0,18,112),
readln(Miasto),
prawda(Miasto),
write("miasto koncowe :"),
field_attr(1,0,15,112),
readln(Cel),
prawda(Cel),
plan(Miasto,Cel,_,_).

/* Inicjacja poszukiwan */  
plan(Miasto,Cel,Wynik,Dystans):-
     eq(Trasa,[Cel]),
     szukaj_trasy(Cel,Miasto,Trasa,Wynik,0,Dystans),
     makewindow(4,15,112,"SZUKANA TRASA",0,0,15,80),
     writelist(Wynik),
     write(" DLUGOSC TRASY :"),
     write(Dystans),
     write(" km"),

/* Dialog z uzytkownikiem */     
     makewindow(5,7,7,"DIALOG",15,0,5,80),
     write("CZY PRACUJEMY DALEJ?  tak PRESS 1"),nl,
     write("                      nie PRESS 0"),nl,
     field_attr(0,0,20,112),
     field_attr(0,21,15,15),
     field_attr(1,21,15,15),
     readreal(Z),
     dial(Z),
     !.
     
dial(P):-NIE=0,P>NIE,okno,!.
dial(_).

/* Sprawdzanie danych wejsciowych z baza danych */
prawda(T):-polozenie(T,_,_),!.
prawda(_):-makewindow(7,7,240,"KOMUNIKAT",15,51,10,29),
           write(" "),nl,
           write("   NIE MA TAKIEGO MIASTA"),nl,
           write("       PRESS ANY KEY"),
           sound(20,392),
           sound(20,349),
           sound(20,466),
           sound(10,523),
           sound(10,698),
           sound(20,587),
           sound(10,523),
           sound(10,698),
           sound(20,587),
           sound(20,466),
           sound(20,523),
           sound(20,392),
           sound(20,349),
           readchar(_),
           okno.
    
                                                             
/* Baza danych programu */
     droga(biala_podlaska,siedlce,83).
     droga(biala_podlaska,lublin,166).
     droga(bialystok,suwalki,140).
     droga(bialystok,lomza,102).
     droga(bialystok,siedlce,160).
     droga(bielsko_biala,krakow,100).
     droga(bielsko_biala,katowice,54).
     droga(bielsko_biala,nowy_sacz,169).
     droga(bydgoszcz,gdansk,160).
     droga(bydgoszcz,elblag,175).
     droga(bydgoszcz,torun,50).
     droga(bydgoszcz,konin,177).
     droga(bydgoszcz,poznan,134).
     droga(bydgoszcz,pila,87).
     droga(chelm,lublin,72).
     droga(chelm,zamosc,83).
     droga(ciechanow,olsztyn,133).
     droga(ciechanow,warszawa,93).
     droga(ciechanow,plock,160).
     droga(ciechanow,torun,187).
     droga(czestochowa,piotrkow_trybunalski,86).
     droga(czestochowa,kielce,122).
     droga(czestochowa,katowice,88).
     droga(czestochowa,opole,96).
     droga(czestochowa,sieradz,120).
     droga(elblag,olsztyn,143).
     droga(elblag,torun,167).
     droga(elblag,gdansk,79).
     droga(gdansk,slupsk,131).
     droga(gorzow_wielkopolski,szczecin,144).
     droga(gorzow_wielkopolski,pila,117).
     droga(gorzow_wielkopolski,poznan,143).
     droga(gorzow_wielkopolski,zielona_gora,118).
     droga(jelenia_gora,zielona_gora,221).
     droga(jelenia_gora,legnica,93).
     droga(jelenia_gora,walbrzych,47).
     droga(kalisz,konin,163).
     droga(kalisz,sieradz,50).
     droga(kalisz,opole,155).
     droga(kalisz,wroclaw,129).
     droga(kalisz,leszno,122).
     droga(kalisz,poznan,137).
     droga(katowice,krakow,78).
     droga(katowice,kielce,170).
     droga(katowice,opole,96).
     droga(kielce,radom,85).
     droga(kielce,tarnobrzeg,164).
     droga(kielce,krakow,132).
     droga(kielce,piotrkow_trybunalski,197).
     droga(konin,wloclawek,134).
     droga(konin,plock,132).
     droga(konin,sieradz,140).
     droga(konin,poznan,100).
     droga(koszalin,slupsk,67).
     droga(koszalin,pila,158).
     droga(koszalin,szczecin,160).
     droga(krakow,tarnow,78).
     droga(krosno,nowy_sacz,100).
     droga(krosno,tarnow,126).
     droga(krosno,rzeszow,93).
     droga(krosno,przemysl,145).
     droga(legnica,zielona_gora,137).
     droga(legnica,wroclaw,65).
     droga(legnica,walbrzych,78).
     droga(leszno,wroclaw,96).
     droga(leszno,poznan,69).
     droga(leszno,zielona_gora,100).
     droga(lublin,zamosc,117).
     droga(lublin,tarnobrzeg,131).
     droga(lublin,radom,129).
     droga(lublin,siedlce,139).
     droga(lomza,ostroleka,42).
     droga(lodz,skierniewice,66).
     droga(lodz,piotrkow_trybunalski,65).
     droga(lodz,sieradz,62).
     droga(lodz,plock,121).
     droga(nowy_sacz,tarnow,88).
     droga(olsztyn,suwalki,225).
     droga(olsztyn,ostroleka,138).
     droga(olsztyn,torun,162).
     droga(opole,wroclaw,82).
     droga(ostroleka,warszawa,115).
     droga(ostroleka,siedlce,125).
     droga(pila,poznan,96).
     droga(pila,slupsk,176).
     droga(piotrkow_trybunalski,skierniewice,77).
     droga(piotrkow_trybunalski,radom,154).
     droga(plock,warszawa,177).
     droga(plock,skierniewice,120).
     droga(plock,wloclawek,108).
     droga(poznan,zielona_gora,137).
     droga(przemysl,rzeszow,87).
     droga(przemysl,zamosc,207).
     droga(radom,warszawa,105).
     droga(radom,siedlce,167).
     droga(radom,tarnobrzeg,161).
     droga(radom,skierniewice,155).
     droga(rzeszow,tarnow,80).
     droga(rzeszow,tarnobrzeg,70).
     droga(rzeszow,zamosc,208).
     droga(siedlce,warszawa,96).
     droga(skierniewice,warszawa,66).
     droga(tarnobrzeg,tarnow,104).
     droga(torun,wloclawek,54).
     droga(walbrzych,wroclaw,80).
     
     polozenie(biala_podlaska,14.3,6.9).
     polozenie(bialystok,14.3,9.8).
     polozenie(bielsko_biala,7.4,0.4).
     polozenie(bydgoszcz,5.7,9.6).
     polozenie(chelm,15.2,4.4).
     polozenie(ciechanow,10.1,8.9).
     polozenie(czestochowa,7.5,3.3).
     polozenie(elblag,8.0,12.6).
     polozenie(gdansk,6.8,13.1).
     polozenie(gorzow_wielkopolski,1.0,8.8).
     polozenie(jelenia_gora,1.6,3.6).
     polozenie(kalisz,5.7,6.0).
     polozenie(katowice,7.3,1.7).
     polozenie(kielce,10.2,3.4).
     polozenie(konin,6.0,7.1).
     polozenie(koszalin,2.7,12.7).
     polozenie(krakow,9.0,1.2).
     polozenie(krosno,12.2,0.1).
     polozenie(legnica,2.3,4.5).
     polozenie(leszno,3.1,6.2).
     polozenie(lublin,13.5,4.6).
     polozenie(lomza,12.4,9.9).
     polozenie(lodz,8.1,5.9).
     polozenie(nowy_sacz,10.4,0).
     polozenie(olsztyn,9.7,11.5).
     polozenie(opole,5.4,3.0).
     polozenie(ostroleka,11.5,9.5).
     polozenie(pila,3.5,9.8).
     polozenie(piotrkow_trybunalski,8.5,5.0).
     polozenie(plock,8.5,8.1).
     polozenie(poznan,3.8,7.7).
     polozenie(przemysl,14.0,0.6).
     polozenie(radom,11.0,4.9).
     polozenie(rzeszow,12.6,1.2).
     polozenie(siedlce,12.9,7.2).
     polozenie(sieradz,6.8,5.4).
     polozenie(skierniewice,9.3,6.5).
     polozenie(slupsk,4.2,13.4).
     polozenie(suwalki,13.8,12.6).
     polozenie(szczecin,0,10.7).
     polozenie(tarnobrzeg,12.0,2.7).
     polozenie(tarnow,10.8,1.1).
     polozenie(torun,6.6,9.4).
     polozenie(walbrzych,2.6,3.2).
     polozenie(warszawa,10.7,7.2).
     polozenie(wloclawek,7.3,8.3).
     polozenie(wroclaw,3.8,4.1).
     polozenie(zamosc,14.7,3.2).
     polozenie(zielona_gora,1.4,6.5).                                                            